#include <ESP8266WiFi.h>
#include "Gsender.h"

#pragma region Globals
const char* ssid = "YOURSSID";                           // WIFI network name
const char* password = "YOURWIFIPASSWORD";                       // WIFI network password
uint8_t connection_state = 0;                    // Connected to WIFI or not
uint16_t reconnect_interval = 10000;             // If not connected wait time to try again
#pragma endregion Globals

const int rPINs[] = {5,4,14,12,13};            //relay pins
int pinCount = 5;                          // Number of elements in above array of relay pins 
String emails[] = {"1111111111@vzwpix.com", "2222222222@vzwpix.com" ,"3333333333@mms.att.net" ,"4444444444@tmomail.net"}; // Email addresses to send notifications to
int numEmails = 4;
long alarm_delay = 600;                    // How long the alarm must last before triggering an email (seconds)
long resend_delay = 86400;                       // How long to wait between emails (seconds)

int numAlarms = 0;
boolean res[sizeof(rPINs)];

Gsender *gsender = Gsender::Instance();    // Getting pointer to class instance
String subject = "Incubator Alarm!";

uint8_t WiFiConnect(const char* nSSID = nullptr, const char* nPassword = nullptr)
{
    static uint16_t attempt = 0;
    Serial.print("Connecting to ");
    if(nSSID) {
        WiFi.begin(nSSID, nPassword);  
        Serial.println(nSSID);
    } else {
        WiFi.begin(ssid, password);
        Serial.println(ssid);
    }

    uint8_t i = 0;
    while(WiFi.status()!= WL_CONNECTED && i++ < 50)
    {
        delay(200);
        Serial.print(".");
    }
    ++attempt;
    Serial.println("");
    if(i == 51) {
        Serial.print("Connection: TIMEOUT on attempt: ");
        Serial.println(attempt);
        if(attempt % 2 == 0)
            Serial.println("Check if access point available or SSID and Password\r\n");
        return false;
    }
    Serial.println("Connection: ESTABLISHED");
    Serial.print("Got IP address: ");
    Serial.println(WiFi.localIP());
    return true;
}

void Awaits()
{
    uint32_t ts = millis();
    while(!connection_state)
    {
        delay(50);
        if(millis() > (ts + reconnect_interval) && !connection_state){
            connection_state = WiFiConnect();
            ts = millis();
        }
    }
}

void setup()
{
    Serial.begin(115200);
    connection_state = WiFiConnect();
    if(!connection_state)  // if not connected to WIFI
        Awaits();          // constantly trying to connect

    
    //if(gsender->Subject(subject)->Send("6267551485@vzwpix.com", "Setup test")) {
    //    Serial.println("Message send.");
    //} else {
    //    Serial.print("Error sending message: ");
    //    Serial.println(gsender->getError());
    //}

    pinMode(LED_BUILTIN, OUTPUT);     // Initialize the LED_BUILTIN pin as an output
    
    for (int thisPin = 0; thisPin < pinCount; thisPin++) {
    pinMode(rPINs[thisPin], INPUT_PULLUP);    // Initialize relay pins as input
    }
        
    digitalWrite(LED_BUILTIN, HIGH);
}

void loop(){
  numAlarms = 0;    // Reset
  
  for (int thisPin = 0; thisPin < pinCount; thisPin++) {
    res[thisPin] = digitalRead(rPINs[thisPin]);    // Read relays into an array
    if (res[thisPin] == HIGH ) numAlarms++;       // Count number of alarming elements
  }

  if (numAlarms > 0) {    // Alarm circuit first trips
      String message = "Alarming state (Devices indicating 1 are alarming):  \n";
      for (int thisPin = 0; thisPin < pinCount; thisPin++) {
        message += "Pin " + String(rPINs[thisPin]) + ": " + String(res[thisPin]) + "\n";
      }
      Serial.println(message);  //Report initial alarm over serial connection
      digitalWrite(LED_BUILTIN, LOW);      
      delay(alarm_delay*1000);                                           // Wait some period of time

      numAlarms = 0;
      for (int thisPin = 0; thisPin < pinCount; thisPin++) {
        res[thisPin] = digitalRead(rPINs[thisPin]);    // Read relays into an array again
        if (res[thisPin] == HIGH ) numAlarms++;       // Count number of alarming elements
      }
      
      if (numAlarms > 0) {  // Check to see if an alarm is still going after waiting for alarm_delay
          
          String message = "Now reporting alarming state (Devices indicating 1 are alarming):  \n";
          for (int thisPin = 0; thisPin < pinCount; thisPin++) {
            message += "Pin " + String(rPINs[thisPin]) + ": " + String(res[thisPin]) + "\n";
          }
          Serial.println(message);  //Report alarm notifications over serial connection

          for (int i = 0; i < numEmails; i++) {
            if(gsender->Subject(subject)->Send(emails[i], message )) {   //Send alarm notifications
              Serial.println("Message send.");
              } else {
              Serial.print("Error sending message: ");
              Serial.println(gsender->getError());
            }
          }
          delay(resend_delay*1000);                                      // Wait some period of time before monitoring again
          
      }
      digitalWrite(LED_BUILTIN, HIGH);
  }
}

